<?php
/*
Name: Thesis Developer Tools
Author: Matt Gross — DIYthemes
Description: Enable Thesis’ developer features with the push of a button.
Version: 1.1
Class: thesis_dev_tools
License: MIT

Copyright 2013 DIYthemes, LLC.
DIYthemes, Thesis, and the Thesis Theme are registered trademarks of DIYthemes, LLC.

Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*/
class thesis_dev_tools extends thesis_box {
	public $type = false;
	private $settings = array();

	protected function translate() {
		$this->title = __('Thesis Dev Tools', $this->_class);
	}

	protected function construct() {
		global $thesis;
		$this->settings = is_array($settings = $thesis->api->get_option($this->_class)) ? $settings : array();
		if (!is_admin() && !empty($this->settings['query']['on']))
			add_action('thesis_hook_after_html', array($this, 'query_test'), 100);
		if (!empty($this->settings['dev_update']['on']))
			add_filter('thesis_dev_updates', '__return_true');
		if (!empty($this->settings['zip']['on']))
			add_filter('thesis_skin_create_zip', '__return_true');
		if (!empty($this->settings['editor_options']['on']))
			add_filter('thesis_editor_box_options', '__return_true');
		if (is_admin() && !empty($_GET['canvas']) && $_GET['canvas'] == $this->_class && !empty($_GET['subsection']) && method_exists($this, ($canvas = 'canvas_' . preg_replace('/\W/', '', $_GET['subsection']))))
			add_action('thesis_admin_canvas', array($this, $canvas));
		add_action('admin_post_t_create_skin', array($this, 'save_skin'));
		add_action('admin_post_t_create_skin_skeleton', array($this, 'create_skeleton'));
		add_action('admin_post_t_import_skeleton_file', array($this, 'import_skeleton'));
	}

	public function admin_init() {
		global $thesis;
		add_action('admin_head', array($this, 'admin_js'));
		wp_enqueue_style('thesis-admin');
		wp_enqueue_style('thesis-popup');
		wp_enqueue_script('thesis-ui', THESIS_JS_URL. '/ui.js', array(), $thesis->version);
	}

	public function admin_ajax() {
		add_action('wp_ajax_save_dev_tools', array($this, 'save'));
		add_action('wp_ajax_check_dev_tools', array($this, 'check'));
	}

	public function admin($message = false) {
		global $thesis;
		// nothing if we're in a subsection
		if (!empty($_GET['subsection']))
			return;
		$updates = $thesis->api->form->fields(array(
			'dev_update' => array(
				'type' => 'checkbox',
				'label' => __('Developer Updates', $this->_class),
				'tooltip' => __('Due to the unstable nature of developer releases, we do NOT recommend using this on a live site. Make sure you understand the risks of using developer updates before turning on this option.', $this->_class),
				'options' => array('on' => __('Use developer updates', $this->_class)))), $this->settings, 't_dev_tools_', '', 10, 3);
		$skins = $thesis->api->form->fields(array(
			'editor_options' => array(
				'type' => 'checkbox',
				'label' => __('Skin Editor Options', $this->_class),
				'options' => array( 'on' => __('Show box options in the Skin Editor', $this->_class)),
				'tooltip' => __('This will allow you to edit Box options in the Skin Editor instead of having to switch back and forth between the Editor and admin side during Skin development.', $this->_class)),
			'zip' => array(
				'type' => 'checkbox',
				'label' => __('Skin Distribution', $this->_class),
				'options' => array('on' => __('Enable active Skin to be downloaded as a .zip file for distribution', $this->_class)))), $this->settings, 't_dev_tools_', '', 10, 3);
		$query = $thesis->api->form->fields(array(
			'query' => array(
				'type' => 'checkbox',
				'label' => __('WP Query Tester', $this->_class),
				'tooltip' => __('This is a great way to test the efficiency of your Skin. Ensure that you are performing as few database queries as possible, and you&#8217;ll end up with a lightning-fast Skin that your users will appreciate.', $this->_class),
				'options' => array('on' => __('Turn on the WP Query Tester', $this->_class)),
				'dependents' => array('on')),
			'queries' => array(
				'type' => 'checkbox',
				'options' => array('on' => __('Output individual queries', $this->_class)),
				'parent' => array('query' => 'on'))), $this->settings, 't_dev_tools_', '', 10, 3);
		$export = $thesis->api->form->fields(array(
			'export' => array(
				'type' => 'checkbox',
				'options' => array(
					'boxes' => sprintf(__('%s Boxes', $this->_class), $thesis->skins->skin['name']),
					'templates' => sprintf(__('%s Templates', $this->_class), $thesis->skins->skin['name']),
					'vars' => sprintf(__('%s Variables', $this->_class), $thesis->skins->skin['name']),
					'css' => sprintf(__('%s CSS', $this->_class), $thesis->skins->skin['name']),
					'css_custom' => sprintf(__('%s Custom CSS', $this->_class), $thesis->skins->skin['name']),
					'_design' => sprintf(__('%s Design Options', $this->_class), $thesis->skins->skin['name']),
					'_display' => sprintf(__('%s Display Options', $this->_class), $thesis->skins->skin['name'])),
				'default' => array(
					'boxes' => true,
					'templates' => true,
					'vars' => true,
					'css' => true,
					'css_custom' => true,
					'_design' => true,
					'_display' => true))), array(), 't_dev_tools_', '', 10, 3);
		$inner =
			$updates['output'].
			"\t\t\t<div class=\"option_item option_field\">\n".
			"\t\t\t\t<input type=\"button\" data-style=\"button action\" name=\"check_dev_tools\" id=\"check_dev_tools\" value=\"". __('Check for Updates', $this->_class). "\" />\n".
			"\t\t\t</div>\n".
			$skins['output'].
			"\t\t\t<div class=\"option_item option_field\">\n".
			"\t\t\t\t<label class=\"list_label\">". __('Create Skin', $this->_class). "</label>".
			"\t\t\t\t<a href=\"". esc_url(add_query_arg(array('page' => 'thesis', 'canvas' => $this->_class, 'subsection' => 'create_skin'), admin_url('admin.php'))). "\" data-style=\"button action\">". __('Create New Skin', $this->_class). "</a>\n".
			"\t\t\t</div>\n".
			"\t\t\t<div class=\"option_item option_field\">\n".
			"\t\t\t\t<label class=\"list_label\">". __('Skin Skeleton Data', $this->_class). " <span class=\"toggle_tooltip\">&#59140;</span></label>".
			"\t\t\t\t<button id=\"t_skeleton_data\" data-style=\"button action\">". __('Export Skeleton Data', $this->_class). "</button>\n".
			"\t\t\t\t<button id=\"t_skeleton_file\" data-style=\"button action\">". __('Import Skeleton Data', $this->_class). "</button>\n".
			"\t\t\t\t<p class=\"tooltip\">". __('Skin Skeleton Data allows you to import the settings from one skin to another. When you export a skeleton data file, it will take the settings from your active skin. You can then take this file and import it (in the dev tools box only) to another skin. It will overwrite (or create) any of the options you selected in the export file. This is useful for creating a base design that you desire to use in future projects.', $this->_class). "</p>\n".
			"\t\t\t</div>\n".
			$query['output'].
			"\t\t\t<input type=\"submit\" data-style=\"button save\" class=\"t_save\" id=\"save_dev_tools\" name=\"save_dev_tools\" value=\"". __('Save Options', $this->_class). "\" />\n".
			"\t\t\t". wp_nonce_field('thesis-dev-tools', '_wpnonce-thesis-dev-tools', true, false). "\n";
		echo (!empty($_GET['result']) && !empty($_GET['message']) ?
			$thesis->api->alert(esc_attr(urldecode($_GET['message'])), 'skin_created', true) : ''),
			"\t\t<h3>", __('Thesis Developer Tools', $this->_class), "</h3>\n",
			"\t\t<form id=\"thesis_dev_tools\" class=\"thesis_options_form t_canvas_left\" method=\"post\" action=\"\">\n",
			$inner,
			"\t\t</form>\n",
			$thesis->api->popup(array(
				'id' => 'skeleton_data',
				'title' => __('Skeleton Data Options', $this->_class),
				'depth' => 2,
				'body' =>
					"<form method=\"post\" action=\"". esc_url(add_query_arg(array('action' => 't_create_skin_skeleton'), admin_url('admin-post.php'))). "\">\n".
					$export['output'].
					"\t<input data-style=\"button action\" type=\"submit\" id=\"t_skeleton_data_submit\" value=\"". __('Download Skeleton File', $this->_class). "\" />\n".
					"\t". wp_nonce_field("{$this->_class}_skeleton_data", '_wpnonce', true, false). "\n".
					"</form>\n")),
			$thesis->api->popup(array(
				'id' => 'skeleton_import',
				'title' => __('Import Skeleton File', $this->_class),
				'depth' => 2,
				'body' =>
					"<form method=\"post\" action=\"". esc_url(add_query_arg(array('action' => 't_import_skeleton_file'), admin_url('admin-post.php'))). "\" enctype=\"multipart/form-data\">\n".
					"\t<input data-style=\"input\" type=\"file\" name=\"t_skeleton_import\" />\n".
					"\t<input data-style=\"button action\" type=\"submit\" id=\"t_skeleton_file_submit\" value=\"". __('Import Skeleton File', $this->_class). "\" />\n".
					"\t". wp_nonce_field("{$this->_class}_skeleton_file", '_wpnonce', true, false). "\n".
					"</form>\n"));
	}

	public function query_test() {
		global $wpdb;
		echo "<p>$wpdb->num_queries queries in ", timer_stop(0, 6), " seconds.</p>\n";
		if (!empty($this->settings['queries']['on']))
			print_r($wpdb->queries);
	}

	public function save() {
		global $thesis;
		$thesis->wp->check('edit_theme_options');
		parse_str(stripslashes($_POST['form']), $form);
		$thesis->wp->nonce($form['_wpnonce-thesis-dev-tools'], 'thesis-dev-tools');
		$save = array();
		foreach ($form as $option => $value)
			if (strpos($option, '_') !== 0)
				$save[$option]['on'] = (bool) $value['on'];
		if (is_array($save) && !empty($save))
			update_option($this->_class, $save);
		else
			delete_option($this->_class);
		echo $thesis->api->alert(__('Developer Tools options saved!', $this->_class), 'dev_tools_saved', true);
		if ($thesis->environment == 'ajax') die();
	}

	public function check() {
		global $thesis;
		$thesis->wp->check('edit_theme_options');
		parse_str(stripslashes($_POST['form']), $form);
		$thesis->wp->nonce($form['_wpnonce-thesis-dev-tools'], 'thesis-dev-tools');
		foreach (array('skins', 'boxes', 'packages', 'core') as $tr)
			delete_transient("thesis_{$tr}_update");
		delete_transient('thesis_callout');
		wp_cache_flush();
		return true;
	}

	public function canvas_create_skin() {
		global $thesis;
		if (!empty($_GET['result']) && $_GET['result'] == 'error') {
			foreach ($_GET['headers'] as $key => $value)
				if (!empty($value))
					$$key = $value;
		}
		$fields = $thesis->api->form->fields(array(
			'name' => array(
				'type' => 'text',
				'width' => 'long',
				'label' => __('Skin Name', $this->_class),
				'tooltip' => __('This is the proper name of your Skin. For instance, My Cool Skin.', $this->_class),
				'default' => isset($name) ? $name : ''),
			'author' => array(
				'type' => 'text',
				'width' => 'long',
				'label' => __('Skin Author', $this->_class),
				'tooltip' => __('This is the Skin author proper name. For instance, Matt Gross.', $this->_class),
				'default' => isset($author) ? $author : ''),
			'description' => array(
				'type' => 'textarea',
				'rows' => 3,
				'label' => __('Skin Description', $this->_class),
				'tooltip' => __('Provide a brief description of your Skin. Try to keep it one paragraph or less.', $this->_class),
				'default' => isset($description) ? $description : ''),
			'version' => array(
				'type' => 'text',
				'width' => 'tiny',
				'label' => __('Skin Version', $this->_class),
				'tooltip' => __('Since this is a new Skin, we recommend starting at 1.0.', $this->_class),
				'default' => isset($version) ? $version : ''),
			'image' => array(
				'type' => 'image',
				'upload_label' => __('Skin Screenshot', $this->_class),
				'tooltip' => __('You may add a screenshot to this skin. This is not a required field.', $this->_class),
				'input_only' => true),
			'class' => array(
				'type' => 'text',
				'width' => 'medium',
				'label' => __('Class Name', $this->_class),
				'tooltip' => sprintf(__('This is the %s class name for your Skin. If you are not sure what to put here, Thesis will suggest a class name based on the Skin Name from above.', $this->_class), $thesis->api->base['php']),
				'default' => isset($class) ? $class : '')), $this->settings, 't_dev_tools_', '', 3, 10);
		echo
			"\t\t<h3>", __('Thesis Developer Tools', $this->_class), "</h3>\n",
			"\t\t<form id=\"thesis_dev_tools\" class=\"thesis_options_form t_canvas_left\" method=\"post\" action=\"", add_query_arg(array('action' => 't_create_skin'), admin_url('admin-post.php')), "\" enctype=\"multipart/form-data\">\n",
			$fields['output'],
			"\t\t\t<input type=\"submit\" data-style=\"button action\" id=\"save_skin_dev_tools\" name=\"save_skin_dev_tools\" value=\"", __('Create Skin', $this->_class), "\" />\n",
			"\t\t\t<a href=\"", add_query_arg(array('page' => 'thesis', 'canvas' => $this->_class), admin_url('admin.php')), "\" data-style=\"button delete\">", __('Cancel', 'thesis'), "</a>\n",
			"\t\t\t", wp_nonce_field('thesis-dev-tools', '_wpnonce-thesis-dev-tools', true, false), "\n",
			"\t\t</form>\n";
	}

	public function save_skin() {
		global $thesis;
		$thesis->wp->check('edit_theme_options');
		$thesis->wp->nonce($_POST['_wpnonce-thesis-dev-tools'], 'thesis-dev-tools');
		extract($_POST); // name, author, description, version, class
		$name = !empty($name) ? sanitize_user($name) : false;
		$author = !empty($author) ? sanitize_user($author) : false;
		$description = !empty($description) ? sanitize_text_field($description) : false;
		$version = !empty($version) ? $version : false;
		$class = !empty($class) ?
			$thesis->api->verify_class_name($this->class_name($class)) :
			$this->class_name($name);
		$headers = compact('name', 'author', 'description', 'version', 'class');
		$errors = array_map('__return_false', array_flip(array_keys($headers, false)));
		if (!empty($errors))
			$query = array(
				'page' => 'thesis',
				'canvas' => $this->_class,
				'subsection' => 'create_skin',
				'result' => 'error',
				'fields' => $errors,
				'headers' => $headers);
		elseif ($this->make_skin($headers))
			$query = array(
				'page' => 'thesis',
				'canvas' => $this->_class,
				'result' => 'success',
				'message' => urlencode(__('Your Skin was created!', $this->_class)));
		else
			$query = array(
				'page' => 'thesis',
				'canvas' => $this->_class,
				'result' => 'error',
				'message' => urlencode(__('Your Skin was not created. Please try again.', $this->_class)));
		wp_redirect(add_query_arg($query, admin_url('admin.php')));
		exit;
	}

	public function make_skin($headers) {
		$result = new thesis_create_skin($headers);
		return $result->result;
	}

	public function create_skeleton() {
		global $thesis;
		if (empty($_POST['export']) || !is_array($_POST['export']))
			wp_die(__('No data was passed. Please press the back button and try again.', $this->_class));
		$thesis->wp->check('edit_theme_options');
		$thesis->wp->nonce($_POST['_wpnonce'], "{$this->_class}_skeleton_data");
		$data = array();
		foreach ($_POST['export'] as $item => $bool)
			$data[$item] = $thesis->api->get_option("{$thesis->skins->skin['class']}_{$item}");
		$data = array_filter($data);
		header('Content-Type: text/plain; charset='. get_option('blog_charset'));
		header('Content-Disposition: attachment; filename="'. str_replace(array('_', ' '), '-', strtolower($thesis->skins->skin['name'])). '-skeleton-data-'. @date('Y\-m\-d\-H\-i'). '.txt"');
		printf('%s', serialize($data));
		exit;
	}

	public function import_skeleton() {
		global $thesis;
		if (empty($_FILES['t_skeleton_import']) || (int) $_FILES['t_skeleton_import']['error'] === 4 || !is_uploaded_file($_FILES['t_skeleton_import']['tmp_name']))
			wp_die(__('No data was passed. Please press the back button and try again.', $this->_class));
		$thesis->wp->check('edit_theme_options');
		$thesis->wp->nonce($_POST['_wpnonce'], "{$this->_class}_skeleton_file");
		if (!is_array($data = unserialize(file_get_contents($_FILES['t_skeleton_import']['tmp_name']))))
			wp_die(__('We were unable to read your Skeleton Data File. Try exporting and importing the file again.', $this->_class));
		$possible = array('boxes', 'vars', 'templates', 'css', 'css_custom', '_design', '_display');
		$definite = array();
		foreach ($possible as $item) {
			if (!empty($data[$item])) {
				update_option("{$thesis->skins->skin['class']}_{$item}", $data[$item]);
				if ($item == 'vars')
					$thesis->skin->_css->restore_vars($data[$item]);
			}
		}
		wp_cache_flush();
		$thesis->skin->_write_css();
		wp_redirect(admin_url("admin.php?page=thesis&canvas=". $this->_class. "&result=success&message=". urlencode(__('Skeleton Data Imported!', $this->_class))));
		exit;
	}

	public function class_name($name = false) {
		global $thesis;
		if (empty($name) || !is_string($name)) return false;
		$name = str_replace(' ', '_', $name);
		$name = preg_replace('/\W/', '', strtolower($name));
		if (!$thesis->api->verify_class_name($name))
			$name = "t_$name";
		return $name;
	}

	public function admin_js() {
?>
<script type="text/javascript">
var thesis_dev_tools;
(function($) {
thesis_dev_tools = {
	init: function() {
		$('#save_dev_tools').on('click', function() {
			thesis_dev_tools.save();
			return false;
		});
		$('#check_dev_tools').on('click', function() {
			thesis_dev_tools.check();
			return false;
		});
		$('#t_skeleton_data').on('click', function() {
			thesis_ui.popup('#popup_skeleton_data');
			return false;
		});
		$('#t_skeleton_file').on('click', function() {
			thesis_ui.popup('#popup_skeleton_import');
			return false;
		});
		$('#t_skeleton_file_submit').on('click', function() {
			if (confirm('Importing Skeleton Data will erase data on your active skin. Are you sure you want to do this?'))
				return true;
			else return false;
		});
<?php if (!empty($_GET['result']) && !empty($_GET['fields'])): ?>
		thesis_dev_tools.error(<?php echo json_encode($_GET['fields']); ?>);
<?php endif; ?>
	},
	save: function() {
		$('#save_dev_tools').prop('disabled', true);
		$.post(ajaxurl, { action: 'save_dev_tools', form: $('#thesis_dev_tools').serialize() }, function(saved) {
			$('#save_dev_tools').prop('disabled', false);
			$('#t_canvas').append(saved);
			$('#dev_tools_saved').css({'right': $('#save_dev_tools').outerWidth()+35+'px'});
			$('#dev_tools_saved').fadeOut(3000, function() { $(this).remove(); });
		});
	},
	check: function() {
		$.post(ajaxurl, { action: 'check_dev_tools', form: $('#thesis_dev_tools').serialize() }, function(saved) {
			window.location.reload();
		});
	},
	error: function(errors) {
		$.each(errors, function(input){
			$('#t_dev_tools_' + input).closest('div.option_field').css({ 'padding' : '10px', 'background' : 'rgba(255, 75, 49, .8)' });
		});
	}
};
$(document).ready(function($){
	thesis_dev_tools.init();
<?php if (!empty($_GET['result']) && !empty($_GET['message'])): ?>
		$('#skin_created').css({'right': $('#save_dev_tools').outerWidth()+35+'px'});
		$('#skin_created').fadeOut(3000, function() { $(this).remove(); });
<?php endif; ?>
});
})(jQuery);
</script>
<?php
	}
}

class thesis_create_skin {
	protected $f, $base, $headers;
	public $result = true;

	public function __construct($headers = array()) {
		if (empty($headers)
		|| !!array_filter(array_diff(array('name', 'author', 'description', 'version', 'class'), array_keys($headers)))
		|| get_filesystem_method() !== 'direct') return false;
		// name, author, description, version, class
		$this->headers = $headers;
		WP_Filesystem();
		$this->f = $GLOBALS['wp_filesystem'];
		$this->base = $this->f->find_folder(THESIS_USER_SKINS);
		$this->dirs();
		$this->files();
		$this->screenshot();
		return $this->result;
	}

	protected function dirs() {
		$this->skin_folder = str_replace('_', '-', strtolower($this->headers['class']));
		if ($this->f->exists($this->base . $this->skin_folder) || !$this->f->mkdir($this->base . $this->skin_folder) || !$this->f->mkdir($this->base . "{$this->skin_folder}/images"))
			$this->result = false;
	}

	protected function files() {
		global $thesis;
		if (!$this->result) return;
		// name, author, description, version, class
		extract($this->headers);
		$contents = sprintf(
			"<?php\n".
			"/*\n".
			"Name: %s\n".
			"Author: %s\n".
			"Description: %s\n".
			"Version: %s\n".
			"Class: %s\n".
			"*/\n".
			"class %s extends thesis_skin {\n".
			"}",
			$this->remove_comments(sanitize_user(stripslashes($name))),
			$this->remove_comments(sanitize_user(stripslashes($author))),
			$this->remove_comments(sanitize_text_field(stripslashes($description))),
			$this->remove_comments($this->sanitize_version($version)),
			$this->remove_comments($thesis->api->verify_class_name($class)),
			$this->remove_comments($thesis->api->verify_class_name($class)));
		$dir = trailingslashit($this->base. $this->skin_folder);
		if (!$this->f->put_contents($dir. 'skin.php', $contents)
		|| !$this->f->put_contents($dir. 'css.css', '')
		|| !$this->f->put_contents($dir. 'custom.php',
			"<?php\n".
			"/*\n".
			"This file is for skin specific customizations. Be careful not to change your Skin’s skin.php file, as that will be upgraded in the future and your work will be lost.\n".
			"If you are more comfortable with PHP, we recommend using the super powerful Thesis Box system to create elements that you can interact with in the Thesis HTML Editor.\n".
			"*/"))
			$this->result = false;
	}

	protected function screenshot() {
		// no image submitted. move along, please.
		if (empty($_FILES['t_dev_tools_image']) || (int) $_FILES['t_dev_tools_image']['error'] === 4)
			return;
		if ($_FILES['t_dev_tools_image']['error'] > 0 || !current_user_can('upload_files')
		|| !is_uploaded_file($_FILES['t_dev_tools_image']['tmp_name']) || !($img_data = @getimagesize($_FILES['t_dev_tools_image']['tmp_name']))
		|| !is_dir($endpoint = trailingslashit($this->base . $this->skin_folder)) || !preg_match('/image\//', $_FILES['t_dev_tools_image']['type'])
		|| @move_uploaded_file($_FILES['t_dev_tools_image']['tmp_name'], $endpoint . 'screenshot.' . strtolower(end(explode('.', $_FILES['t_dev_tools_image']['name'])))) === false)
			$this->result = false;
		else
			$this->result = true;
	}

	protected function sanitize_version($version) {
		preg_match_all('/\w|\./', $version, $matches);
		return str_replace('_', '', implode('', $matches[0]));
	}

	private function remove_comments($text) {
		return preg_replace('/\*\//', '', $text);
	}
}