<?php
// ---- Start ACF Functions ----

function my_acf_options_page_settings($settings) {	
	$settings['title'] = 'Theme Options';
	$settings['pages'] = array('Site Settings', 'Home Page', 'Blog Authors', 'Sidebar');
	return $settings;	
}
add_filter('acf/options_page/settings', 'my_acf_options_page_settings');

function remove_acf_menu() {
    $admins = array('webmaster');
    $current_user = wp_get_current_user();
    if(!in_array($current_user->user_login, $admins)) { remove_menu_page('edit.php?post_type=acf'); }
}
 
add_action( 'admin_menu', 'remove_acf_menu', 999 );

// ---- End ACF Functions ----
// ---- Start Nav Menus ----

function custom_wp_nav_menu($var) {
	return is_array($var) ? array_intersect($var, array(
		'current_page_item',
		'first',
		'last'
	)) : '';
}
add_filter('nav_menu_css_class', 'custom_wp_nav_menu');
add_filter('nav_menu_item_id', 'custom_wp_nav_menu');
add_filter('page_css_class', 'custom_wp_nav_menu');

function strip_empty_classes($menu) {
    $menu = preg_replace('/ class=""| class="sub-menu"/','',$menu);
    return $menu;
}
add_filter ('wp_nav_menu','strip_empty_classes');

function first_last($items) {
    $items[1]->classes[] = 'first';
    $items[count($items)]->classes[] = 'last';
    return $items;
}
add_filter('wp_nav_menu_objects', 'first_last');

add_theme_support( 'menus' );
register_nav_menus( array(
	'Primary' => 'Primary',
	'Mobile' => 'Mobile'	
));

// ---- End Nav Menus ----
// ---- Start Page Number & Custom Title ----

function get_page_number() {
	if (get_query_var('paged')) { return ' | ' . __( 'Page ' , 'thesis') . get_query_var('paged'); }
}  
function use_title($default_title) {
    $title = '';
	$author = get_userdata( get_query_var('author'));
    if (is_front_page()) { $title = get_bloginfo('name') . ' | ' . get_bloginfo('description') . ' | ' . 'Home'; }
	elseif (is_home()) { $title = get_bloginfo('name') . ' | ' . get_page_number() . 'Blog' ; }
	elseif (is_single()) { $title = get_bloginfo('name') . ' | ' . single_post_title('',FALSE)  ; }
	elseif (is_page()) { $title = get_bloginfo('name') . ' | ' . get_page_number() . single_post_title('',FALSE) ; }
	elseif (is_category()) { $title = get_bloginfo('name') . ' | ' . get_page_number() . single_cat_title('',FALSE) ; }
	elseif (is_tag()) { $title = get_bloginfo('name') . ' | ' . get_page_number() . single_tag_title('',FALSE) ; }
	elseif (is_author()) { $title = get_bloginfo('name') . ' | Articles By ' . $author->display_name; }
	elseif (is_day()) { $title = get_bloginfo('name') . ' | ' .  get_the_date('d F Y'); }
	elseif (is_month()) { $title = get_bloginfo('name') . ' | ' .  get_the_date('F Y'); }
	elseif (is_year()) { $title = get_bloginfo('name') . ' | ' .  get_the_date('Y'); }
	elseif (is_search()) { $title = get_bloginfo('name') . ' | Search Results For ' . the_search_query(); }
	elseif (is_404()) { $title = get_bloginfo('name') . ' | Page Not Found'; }
	else { $title = get_bloginfo('name') . ' | ' .get_bloginfo('description'); }
    return $title;	
}
add_filter('thesis_title_tag', 'use_title');  

// ---- End Page Number & Custom Title ----
// ---- Start Meta Data ----

function meta () { ?>
	<meta name="viewport" content="width=device-width, initial-scale=1, maximum-scale=1">
	<meta name="revisit-after" content="7 days">
	<meta name="abstract" content="Idea Driven Organization - Dean Schroeder & Alan Robinson">
	<meta name="author" content="John Marcello">
	<meta name="copyright" content="Copyright &copy; 2014 -  Dean Schroeder & Alan Robinson">
	<link rel="stylesheet" type="text/css" media="screen" href="<?php echo(THESIS_USER_SKINS_URL); ?>/mobile-first/custom.css">
	<link href='http://fonts.googleapis.com/css?family=Open+Sans+Condensed:300,300italic,700|Open+Sans:300italic,400italic,600italic,700italic,800italic,400,300,600,700,800&subset=latin,latin-ext' rel='stylesheet' type='text/css'>
	<link href='http://fonts.googleapis.com/css?family=Source+Sans+Pro:400,600,700,900,400italic,600italic,700italic,900italic' rel='stylesheet' type='text/css'>
<?php }
add_action('wp_head','meta');

// ---- End Meta Data ----
// ---- Start Preload ----

function pre_load () { ?>
	<script src="http://ajax.googleapis.com/ajax/libs/jquery/2.0.3/jquery.min.js" type="text/javascript"></script>
	<script src="<?php echo(THESIS_USER_SKINS_URL); ?>/mobile-first/scripts/cufon-yui.js" type="text/javascript"></script>
	<script src="<?php echo(THESIS_USER_SKINS_URL); ?>/mobile-first/scripts/myriad.js" type="text/javascript"></script>
	<script type="text/javascript">
		Cufon.replace('.myriad_regular', { fontFamily: 'Myriad Pro Regular', hover: true }); 
		Cufon.replace('.myriad_semibold', { fontFamily: 'Myriad Pro Semibold', hover: true }); 
		Cufon.replace('.myriad_semibold_italic', { fontFamily: 'Myriad Pro Semibold Italic', hover: true }); 
		Cufon.replace('.myriad_bold', { fontFamily: 'Myriad Pro Bold', hover: true });
	</script>
	<script src="<?php echo(THESIS_USER_SKINS_URL); ?>/mobile-first/scripts/jquery.p7EHCscripts.js" type="text/javascript"></script>
	<script src="<?php echo(THESIS_USER_SKINS_URL); ?>/mobile-first/scripts/jquery.caroufredsel.js" type="text/javascript"></script>
	<script src="<?php echo(THESIS_USER_SKINS_URL); ?>/mobile-first/scripts/jquery.reveal.js" type="text/javascript"></script>
	<script>
  		(function(i,s,o,g,r,a,m){i['GoogleAnalyticsObject']=r;i[r]=i[r]||function(){
  		(i[r].q=i[r].q||[]).push(arguments)},i[r].l=1*new Date();a=s.createElement(o),
  		m=s.getElementsByTagName(o)[0];a.async=1;a.src=g;m.parentNode.insertBefore(a,m)
  		})(window,document,'script','//www.google-analytics.com/analytics.js','ga');
  		ga('create', 'UA-47683350-1', 'idea-driven.com');
  		ga('send', 'pageview');
	</script>
<?php if(is_single()) { // Share This ?>
	<script type="text/javascript" src="http://w.sharethis.com/button/buttons.js"></script>
	<script type="text/javascript">stLight.options({publisher: "ur-2baa7a22-e72e-7e13-ec77-9b8919ba368", doNotHash: false, doNotCopy: false, hashAddressBar: false});</script>
<?php }
}
add_action('wp_head','pre_load');

// ---- End Preload ----
// ---- Start Mobile Area ----

function mobile_area() { ?>

	<div class="inner mobile">
		<div id="nav-mobi">
			<div class="nav_bar">
            	<div class="flip"></div>
				<div class="panel" style="overflow: hidden; display: none;">
					<?php wp_nav_menu(array(
						'theme_location'  => 'Mobile', 
						'container'       => 'div',
						'container_id'    => 'nav', 
						'menu_class'      => 'menu', 
						'echo'            => true,
						'depth'           => 0,
						)); 
					?>
				</div>
            </div>
            <div class="social">
            	<a target="_blank" href="https://www.facebook.com/"><img src="<?php echo(THESIS_USER_SKINS_URL); ?>/mobile-first/images/icon_facebook.png" alt="facebook" /></a>
				<a href="<?php bloginfo('url'); ?>/contact"><img src="<?php echo(THESIS_USER_SKINS_URL); ?>/mobile-first/images/icon_mail.png" alt="contact" /></a>
				<a href="<?php bloginfo('url'); ?>/feed"><img src="<?php echo(THESIS_USER_SKINS_URL); ?>/mobile-first/images/icon_rss.png" alt="rss" /></a>
            </div>
            <div class="clear"></div>
		</div>
		<script>
		$(document).ready(function($) {
			$(".flip").click(function(){
			$(".panel").slideToggle("fast");
  			});
		});
		</script>
	</div>
	<div class="clear"></div>
	
<?php }

add_filter('thesis_hook_container_mobile_wrapper_top', 'mobile_area');

// ---- End Mobile Area ----
// ---- Start Header Area ----

function header_area() { ?>

	<div class="inner screen">
    	<table border="0" cellpadding="0" cellspacing="0" width="100%" align="center" class="tbl_header">
        	<tr>
            	<td class="logo">
                	<h1>
                    	<a title="Home" href="<?php bloginfo('url'); ?>/">
                    		<img class="flex" src="<?php the_field('site_logo','options'); ?>" alt="<?php bloginfo('name'); ?>" />
                   		</a>
                    </h1>
				</td>
                <td class="grid">
                	<div class="social">
                    	<?php 
						if( have_rows('social_media', 'options')):
						while ( have_rows('social_media', 'options') ) : the_row(); ?>
					        <a <?php the_sub_field('social_media_target'); ?>href="<?php the_sub_field('social_media_link'); ?>">
                            	<img src="<?php the_sub_field('social_media_icon'); ?>" alt="" />
                            </a>
                        <?php endwhile;
						else :
						endif;
 						?>
            		</div>
					<?php wp_nav_menu ( array ( 
						'menu'            => 'Primary', 
						'container'       => 'div',
						'container_id'    => 'nav', 
						'menu_class'      => 'menu', 
						'echo'            => true,
						'depth'           => 0,
						)); 
					?>
                </td>
            </tr>
        </table>
    </div>
	<? }
	
add_filter('thesis_hook_container_header_wrapper_top', 'header_area');

// ---- End Header Area ----
// ---- Start Slider Area ----

function slider_area() { ?>
	<div class="inner">
       	<div class="box">
			<div class="image_carousel">
				<div id="foo1">
                <?php 
					if( have_rows('slides', 'options')):
					while ( have_rows('slides', 'options')) : the_row(); ?>
						<a href="<?php the_sub_field('slider_image_link'); ?>">
                            <img src="<?php the_sub_field('slider_image'); ?>" alt="<?php the_sub_field('slider_image_alt_text'); ?>" />
                    	</a>
                    <?php endwhile;
					else :
					endif;
 				?>
              	</div>
				<div class="clearfix"></div>
				<a class="prev" id="foo1_prev" href="#"><span>prev</span></a>
				<a class="next" id="foo1_next" href="#"><span>next</span></a>
            </div>
		</div>
	</div>
	<div class="clear"></div>
	
	<?php }
add_filter('thesis_hook_container_slider_wrapper_top', 'slider_area');

// ---- End Slider Area ----
// ---- Start Subscribe Area ----

function subscribe_area() { ?>
	<div class="inner">
    	<table border="0" cellpadding="0" cellspacing="0" width="100%" align="center">
        	<tr>
        		<td style="vertical-align: middle;">
                	<div class="top"><h2><?php the_field('subscribe_call', 'options'); ?></h2></div>
    				<div class="bottom"><?php the_field('subscribe_form', 'options'); ?></div>
                </td>
        	</tr>
        </table>

    </div>
	<div class="clear"></div>
	
	<? }
add_filter('thesis_hook_container_subscribe_wrapper_top', 'subscribe_area');

// ---- End Subscribe Area ----
// ---- Start Alan Area ----

function alan_area() { ?>
	<div class="inside">
    	<div class="inner">
        	<div class="top myriad_bold"><h2><?php the_field('features_call', 'options'); ?></h2></div>
			<div class="left p7ehc-a">
        		<img class="screen" src="<?php the_field('feature_top_image', 'options'); ?>" alt="Meet Alan Robinson" />
        	</div>
       	 	<div class="right p7ehc-a">
        		<h3 class="author_title">Dr. Alan G. Robinson</h3>
                <div class="author_text">
                    <img class="mobile" src="<?php the_field('feature_top_mobile_image', 'options'); ?>" alt="Meet Alan Robinson" />
					<span><?php the_field('feature_top_text', 'options'); ?></span>
                </div>
                <div class="author_button"><a href="http://alanrobinson.com">Visit Dr. Robinson's Website</a></div>
                <div class="author_connect">
                	<table border="0" cellpadding="0" cellspacing="0" width="100%" class="tbl_connect">
                    	<tr>
                        	<td class="title">Connect With Alan:</td>
                            <td class="social">            			
							<?php 
								if(have_rows('feature_top_social', 'options')):
								while (have_rows('feature_top_social', 'options')) : the_row(); ?>
					        		<a href="<?php the_sub_field('feature_top_social_media_link'); ?>" <?php the_sub_field('feature_top_social_media_target'); ?>>
                            			<img src="<?php the_sub_field('feature_top_social_media_icon'); ?>" alt="" />
                            		</a>
                        		<?php endwhile;
								else :
								endif;
 							?>
                            </td>
                        </tr>
                    </table>
                </div>
            </div>
    	</div>
	</div>
	<div class="clear"></div>
	<? }
add_filter('thesis_hook_container_alan_wrapper_top', 'alan_area');

// ---- End Alan Area ----
// ---- Start Dean Area ----

function dean_area() { ?>
	
    <div class="inside">
    	<div class="inner">
			<div class="left p7ehc-b">
        		<h3 class="author_title">Dr. Dean M. Schroeder</h3>
                <div class="author_text">
                    <img class="mobile" src="<?php the_field('feature_bottom_mobile_image', 'options'); ?>" alt="Meet Dean Schroeder" />
					<span><?php the_field('feature_bottom_text', 'options'); ?></span>
                </div>
                <div class="author_button"><a href="http://deanmschroeder.com">Visit Dr. Schroeder's Website</a></div>
                <div class="author_connect">
                	<table border="0" cellpadding="0" cellspacing="0" width="100%" class="tbl_connect">
                    	<tr>
                        	<td class="title">Connect With Dean:</td>
                            <td class="social">            			
							<?php 
								if(have_rows('feature_bottom_social', 'options')):
								while (have_rows('feature_bottom_social', 'options')) : the_row(); ?>
					        		<a href="<?php the_sub_field('feature_bottom_social_media_link'); ?>" <?php the_sub_field('feature_bottom_social_target'); ?>>
                            			<img src="<?php the_sub_field('feature_bottom_social_media_icon'); ?>" alt="" />
                            		</a>
                        		<?php endwhile;
								else :
								endif;
 							?>
                            </td>
                        </tr>
                    </table>
                </div>
        	</div>
       	 	<div class="right p7ehc-b">
        		<img class="screen"src="<?php the_field('feature_bottom_image', 'options'); ?>" alt="Meet Dean Schroeder" />	
            </div>
    	</div>
	</div>
    <div class="clear"></div> 
	<? }
add_filter('thesis_hook_container_dean_wrapper_top', 'dean_area');

// ---- End Dean Area ----
// ---- Start Feeds Area ----

function feeds_area() { ?>
	
    <div class="inside">
    	<div class="inner">
			<div class="top myriad_bold"><h2>Start Building Your Idea Driven Organization</h2></div>
			<div class="left p7ehc-c">
				<div class="feed_title"><h3>Read The Lastest Blog Articles</h3></div>
            	<div class="feed_container">
            	<?php 
				// ---- Start Loop ----
				$custom_loop = new WP_Query('showposts=3&orderby=date');
    			$output = "";
				if ( $custom_loop->have_posts() ) : 
        		// ---- open box wrapper ----
				$output .= "<div>\n";	 
        		while ( $custom_loop->have_posts() ) : $custom_loop->the_post(); 
				// ---- load post data ----
            	global $post; 
				// ---- prepare necessary variables ----
            	$img = "";
				$post_url = get_permalink($post->ID);
            	$post_title = get_the_title($post->ID);
            	$post_month = get_the_time( 'M', ($post->ID));
				$post_date = get_the_time( 'd', ($post->ID));
				$post_year = get_the_time( 'Y', ($post->ID));
				$a_title = esc_attr($post_title);
            	$a_before = "<a href=\"{$post_url}\">";
            	$a_after = "</a>";
				$author_name = get_the_author_meta( 'first_name' );
				$author_url = get_author_posts_url(get_the_author_meta('ID'));
            	$author_before = "<a href=\"{$author_url}\">";
            	$author_after = "</a>";
				$category = get_the_category(); 
				$cat_before = '<a href="' . get_category_link($category[0]->term_id ) . '">';
				$post_cat = $category[0]->cat_name;
				$cat_after = "</a>";
				$num_comments = get_comments_number();
				if ( $num_comments == 0 ) { $comments = 'No Comments'; } 
				elseif ( $num_comments > 1 ) { $comments = $num_comments . ' Comments'; }
				else { $comments = '1 Comment';	}
				$post_comments = '<a href="'.get_comments_link().'">'.$comments.'</a>';
				$post_excerpt = get_the_excerpt($post->ID);
				$post_more = 'Read More';
            	// ---- start html constructor ----
				$output .= "<div class=\"feed_master\">\n";
				$output .= "<table class=\"feed_tbl\">\n";	 
				$output .= "<tr>\n"; 
				$output .= "<td class=\"feed_time\">\n"; 
				$output .= "<div class=\"feed_square feed_month\">\n{$post_month} {$post_date}</div>\n"; 
				$output .= "<div class=\"feed_square feed_date\">\n{$post_year}</div>\n"; 
				$output .= "</td>\n";
				$output .= "<td class=\"feed_content\">\n"; 
				$output .= "<div class=\"feed_headline\">\n";
				$output .= "<h4 class=\"feed_title\">{$a_before}{$post_title}{$a_after}</h4>\n";
				$output .= "</div>\n";
				$output .= "<span class=\"meta_author\">By {$author_before}{$author_name}{$author_after}&nbsp;&nbsp;//&nbsp;&nbsp;</span>\n";
				$output .= "<span class=\"meta_cat\">{$cat_before}{$post_cat}{$cat_after}&nbsp;&nbsp;//&nbsp;&nbsp;</span>\n";								
				$output .= "<span class=\"meta_comments\">{$post_comments}</span>\n";								  
				$output .= "</td>\n";
				$output .= "</tr>\n";
				$output .= "<tr>\n";
				$output .= "<td colspan=\"2\" class=\"feed_text\">\n"; 
				$output .= "<div class=\"feed_excerpt\">\n{$post_excerpt}</div>\n"; 
				$output .= "</td>\n";
				$output .= "</tr>\n";
				$output .= "<tr>\n";
				$output .= "<td colspan=\"2\" class=\"feed_button\">\n"; 
				$output .= "<div class=\"feed_more\">{$a_before}{$post_more}{$a_after}</div>\n"; 
				$output .= "</td>\n";
				$output .= "</tr>\n";
				$output .= "</table>\n";
				$output .= "</div>\n";
				endwhile;
				$output .= "</div>\n"; 
    			endif;
				wp_reset_query();
    			wp_reset_postdata();
    			echo $output; 
				?>
            	</div>
       		</div>
			<div class="right p7ehc-c">
            	<table border="0" cellpadding="0" cellspacing="0" align="center" width="100%" class="tbl_preview">
                	<tr>
                    	<td class="p7ehc-c">
                        	<div class="title carto_bold">Get The Book Today!</div>
                            <div class="subtitle carto_bold">Learn How To Unlock The Incredible Power In Your Employees' Ideas.</div>
                            <div><img class="flex" src="<?php echo(THESIS_USER_SKINS_URL); ?>/mobile-first/images/book_idea_driven.png" alt="Idea Driven Organization" /></div>
                            <div class="margin-bottom: 20px; ">
                            	<div class="links read"><a href="<?php echo bloginfo('url'); ?>/landing/idea-driven-sample-chapter">Read A Sample Chapter</a></div>
                            	<div class="links buy"><a target="_blank" href="http://www.amazon.com/The-Idea-Driven-Organization-Unlocking-Bottom-Up/dp/1626561230">Pre-Order On Amazon</a></div>
                            	<div class="links bulk"><a target="_blank" href="http://800ceoread.com/products/idea_driven_organization-alan_g_robinson_dean_m_schroeder-english">Bulk Order On 800 CEO Read</a></div>
                            	<div class="links reviews"><a href="<?php echo bloginfo('url'); ?>/about-idea-driven-organization#reviews">Read Book Reviews</a></div>
                            </div>
                        </td>
                    </tr>
                </table>
            </div>		
			<div class="clear"></div>
        </div>
	</div>
    
	<? }
add_filter('thesis_hook_container_feeds_wrapper_top', 'feeds_area');

// ---- End Feeds Area ----
// ---- Start Title Area ----

function title_area() { ?>
	
    <table border="0" cellpadding="0" cellspacing="0" align="center" width="100%" class="tbl_title">
    	<tr>
        	<td class="spacer_left"><span class="indent"></span></td>
            <td class="spacer_center">
				<h2 class="title">
				<?php
					global $post;
					if(is_home() || is_single()) { ?>The Idea Driven Blog<?php }
					elseif(is_author()) { ?>Posts By: &nbsp;<?php the_author_meta( 'display_name' ); ?><?php }
					elseif(is_category()) { ?>Category:&nbsp;<?php single_cat_title(); }
					elseif(is_tag()) { ?>Tag:&nbsp;<?php single_tag_title(); }
					elseif(is_search()) { printf( __( 'Search Results for: %s' ), '<span>' . get_search_query() . '</span>'); }
					else { the_title(); }
		 			wp_reset_postdata(); 
		 		?>
			</h2>
            </td>
    		<td class="spacer_right"><span class="indent"></span></td>
        </tr>
    </table>
	
	<? }

add_filter('thesis_hook_container_title_area_top', 'title_area');

// ---- End Title Area ----
// ---- Start Contact Area ----

function contact_area() { ?>
	
    <div class="summary">
    	<table border="0" cellpadding="0" cellspacing="0" align="center" width="100%" class="tbl_contact">
        	<tr>
            	<td align="center" colspan="2">
                	<a href="mailto:info@idea-driven.com">
                    	<div><img class="icon" src="<?php echo(THESIS_USER_SKINS_URL); ?>/mobile-first/images/icon_email.png" alt="" /></div>
                    	<div class="email" >info@idea-driven.com</div>
                    </a>
                </td>
            </tr>
            <tr>
            	<td class="info info_left">
                	<div class="name">Dr. Alan G. Robinson</div>
                    <div class="data">51 Bears Den Drive</div>
                    <div class="data">Sunderland, MA 01375</div>
                    <div class="data"><a href="mailto:alan@idea-driven.com">alan@idea-driven.com</a></div>
                </td>
                <td class="info info_right">
                	<div class="name">Dr. Dean M. Schroeder</div>
                    <div class="data">398 Westchester Lane</div>
                    <div class="data">Valparaiso, IN 46385</div>
                    <div class="data"><a href="mailto:dean@idea-driven.com">dean@idea-driven.com</a></div>
                </td>
            </tr>
        </table>
    </div>
	<img class="flex" src="<?php echo(THESIS_USER_SKINS_URL); ?>/mobile-first/images/bg_contact.png" alt="" />
	
	<? }

add_filter('thesis_hook_container_contact_area_top', 'contact_area');

// ---- End Contact Area ----
// ---- Start Subtitle Area ----

function subtitle_area() { ?>
	<div class="inner">
    	<table border="0" cellpadding="0" cellspacing="0" width="100%" align="center">
        	<tr>
        		<td style="vertical-align: middle;">
                	<div class="top"><h2>Use This Form To Send A Message</h2></div>
                </td>
        	</tr>
        </table>

    </div>
	<div class="clear"></div>
	
	<? }
add_filter('thesis_hook_container_subtitle_wrapper_top', 'subtitle_area');

// ---- End Subscribe Area ----
// ---- Start Alan Area ----

// ---- Start Footer Area  ----

function footer_area(){ ?>
	
	<div class="inner">
		<div class="left">
			<?php the_field('copyright_text','options'); ?> &copy; <?php echo date("Y"); ?>
       		<?php if(get_field('attribution_text', 'options') && get_field('attribution_link', 'options')) : ?>
        	<span class="separator">&nbsp;|&nbsp;</span>
        	<a href="<?php the_field('attribution_link','options'); ?>" target="_blank"><?php the_field('attribution_text','options'); ?></a>
            <?php endif; ?>
        </div>
		<div class="right foot">
            <?php 
				if( have_rows('footer_links', 'options')):
				while ( have_rows('footer_links', 'options') ) : the_row(); ?>
					<a href="<?php echo bloginfo('url'); ?>/<?php the_sub_field('footer_link'); ?>"><?php the_sub_field('footer_link_text'); ?></a>
                    <span class="separator"> &nbsp;|&nbsp;</span>
                <?php endwhile;
				else :
				endif;
 			?>
        </div>
		<div class="clear"></div>
	</div>  
    
	<?php 
}
add_filter('thesis_hook_container_footer_wrapper_top', 'footer_area');

// ---- End Footer Area  ----
// ---- Start Postload ----

function front() {
	
	if(is_front_page()) { ?>
	
	<script type="text/javascript">
	$("#foo1").carouFredSel({
		responsive : true,
		items: {
			visible: 1,
			minimum: 1,
			width: 960,
			height: "52.5%"
		},
		scroll: {
			fx: "fade",
			duration: 2500,
			pauseOnHover: true
		},
		auto: 7000,
		prev  : "#foo1_prev",
		next  : "#foo1_next"
	});
	</script> 
	<?php }
}
		
add_filter('thesis_hook_after_html', 'front');

function post_load () { ?>

	<script type="text/javascript"> Cufon.now();</script>
	<script type="text/javascript">
  		jQuery(window).bind('resize', (function() {
    		var timer;
    		return function() {
      		clearTimeout(timer);
      		timer = setTimeout(function() {
        		Cufon.refresh();
      		}, 50); // possibly more than 50
    		};
  		})());
	</script>
	<script>
 		$(document).ready(function(){
    	// Target your .container, .wrapper, .post, etc.
    	$(".format_text").fitVids();
  		});
	</script>
	<script src="<?php echo(THESIS_USER_SKINS_URL) ?>/mobile-first/scripts/fitvids.js" type="text/javascript"></script>
    <script type="text/javascript">$(".feed_master").last().addClass("last");</script>
    <script type="text/javascript">$("li.cat-item").last().addClass("last");</script>
    <script type="text/javascript">$(".tbl_praise tr").last().addClass("last");</script>	
    <script type="text/javascript">$(".foot span").last().addClass("last");</script>	
	<?php }
	
add_filter('thesis_hook_after_html', 'post_load');

// ---- End Postload ----
// ---- Start Custom Excerpt Area  ----

function custom_wp_trim_excerpt($text) {
	
$allowed_tags = '<div>,<span>'; 
$raw_excerpt = $text;
if ( '' == $text ) {
    $text = get_the_content('');
    $text = strip_shortcodes( $text );
    $text = apply_filters('the_content', $text);
    $text = str_replace(']]>', ']]&gt;', $text);
	$text = preg_replace(" (\[.*?\])",'',$text);
	$text = strip_tags($text, $allowed_tags);
if(is_home() || is_archive()) { $text = substr($text, 0, 420); } 
else { $text = substr($text, 0, 220); }
$text = substr($text, 0, strripos($text, " "));
$text = trim(preg_replace( '/\s+/', ' ', $text));
$text = rtrim($text, '.');
$text = rtrim($text, ',');
if(is_home() || is_archive()) { return $text.'&nbsp;&nbsp;&raquo;&nbsp;&nbsp;<a class="read_link" href="' . get_permalink() . '">Read More</a>'; }
else { return $text.'...'; }
}
return apply_filters('wp_trim_excerpt', $text, $raw_excerpt);
}	
remove_filter('get_the_excerpt', 'wp_trim_excerpt');
add_filter('get_the_excerpt', 'custom_wp_trim_excerpt');

// ---- End Postload ----
// ---- End ----

?>